@@ -1,7 +1,9 @@
 from __future__ import annotations
 
+import copy
 import uuid
 from typing import Any, Collection, List, Union
+from abc import ABC, abstractmethod
 
 from returns.result import Failure, Result, Success
 
@@ -12,7 +14,7 @@ Success = Success
 Failure = Failure
 
 
-class Task:
+class Task(ABC):
     """
     This is a base class for all tasks. All new task classes should inherit from this class.
     The task instance should encapsulate all the logic and data needed to execute the task.
@@ -32,30 +34,32 @@ class Task:
     requirements: List[str] = []
 
     # this variable would be overwritten before task start with results of previous tasks
-    previous_steps: List[Union[Result, Collection[Result]]] = []
+    previous_steps: List[Union[Result[Any, Any], Collection[Result[Any, Any]]]] = []
 
-    def __call__(self):
-        return self.run()
-
-    def __str__(self):
-        return self.name
-
-    def add_requirement(self, command: str) -> Task:
-        """
-        This method adds a requirement to the task.
-        :param command:
-        :return:
-        """
-        self.requirements.append(command)
-        return self
-
-    def __init__(self):
+    def __init__(self) -> None:
         """
         This is a constructor for the task. Any variables (state) that `run` method should use should be provided here.
         Please, do not forget to call `super().__init__()` in your implementation.
         """
         self.name = str(uuid.uuid4())  # Each task should have a name
 
+    def __call__(self) -> Any:
+        return self.run()
+
+    def __str__(self) -> str:
+        return self.name
+
+    def add_requirement(self, command: str) -> Task:
+        """
+        This method adds a requirement to the local requirements of the task.
+        :param command:
+        :return:
+        """
+        self.requirements = copy.deepcopy(self.requirements)  # make it instance-specific
+        self.requirements.append(command)
+        return self
+
+    @abstractmethod
     def run(self) -> Any:
         """
         ## This method is to be overridden by your implementation. ##
@@ -69,14 +73,23 @@ class Task:
         raise NotImplementedError
 
 
-class TaskDispatcher:
+class TaskDispatcher(ABC):
     """
-    This class is a wrapper for several tasks that are designed to implement the same functionality for different
-    architectures, platforms, etc. It is designed to be used as a base class for your task dispatcher.
+    This class is a wrapper for several tasks that are designed to implement the same functionality
+    but depend on node attributes. Most often you either want to use a specific
+    implementation for a specific architecture (e.g., different Tasks for Windows and Linux),
+    or instantiate a task with some specific parameters for a specific node (e.g., node-specific IP address).
+    You should implement your own TaskDispatcher class and override the dispatch method.
 
-    Dispatching is done by calling the dispatch method. This method should return the proper task for the node
-    given node information (such as architecture, platform, etc).
+    Dispatching is done by calling the dispatch method that you should implement.
     """
 
+    @abstractmethod
     def dispatch(self, node: Node) -> Task:
+        """
+        This method takes a node and should return and instance of the task that is designed to be executed on this node.
+        The instance could depend on the node information (such as architecture, platform, properties, etc).
+        :param node: Node object
+        :return: Task object
+        """
         raise NotImplementedError
